# -*- coding: utf-8 -*-S
#!/usr/bin/env python

# embedding_in_qt4.py --- Simple Qt4 application embedding matplotlib canvases
#
# Copyright (C) 2005 Florent Rougon
#               2006 Darren Dale
#
# This file is an example program for matplotlib. It may be used and
# modified with no restriction; raw copies as well as modified versions
# may be distributed without limitation.

from __future__ import unicode_literals
import sys, os, random, time
from PyQt4 import QtGui, QtCore

from osgeo import gdal, gdalconst, ogr

from numpy import arange, sin, pi
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure

progname = os.path.basename(sys.argv[0])
progversion = "0.1"

import string
import numpy.ma as ma
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle

class MyMplCanvas(FigureCanvas):
    __pyqtSignals__ = ("valueChanged()")
    
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""
    def __init__(self, parent=None, width=5, height=4, dpi=100):
       
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        # We want the axes cleared every time plot() is called

        self.axes.hold(False)

        FigureCanvas.__init__(self, fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

        self.two_min = 0
        self.ninety_eight_max = 0
        self.x_min = 0
        self.x_max = 0
        self.change_min = True


    def get_GDAL_histogram( self, image, band ):
        """
        From the given binary image, compute histogram and return the number of 1
        """
        histogram = []
        dataset = gdal.Open(str(image), gdal.GA_ReadOnly)
        if dataset is None:
            print "Error : Opening file ", image
        else :
            band = dataset.GetRasterBand(band)
            rasterMin, rasterMax = band.ComputeRasterMinMax()
            nbVal = rasterMax - rasterMin
            if nbVal < 1 :
                return 0
            #get the histogram of the given raster
            histogram = band.GetHistogram(rasterMin, rasterMax+1, int(nbVal+1), approx_ok = 0)
            
            # get 2 - 98 %
            #taking the size of the raster
            sizeX = dataset.RasterXSize
            sizeY = dataset.RasterYSize
            nb_pixels = sizeX * sizeY
            
            nb_pixels_2 = int(float(nb_pixels * 0.02))
            nb_pixels_98 = int(float(nb_pixels * 0.98))
            
            print "nb_pixels_2, nb_pixels_98", nb_pixels_2, nb_pixels_98
            
            # catch the i where cum_hist(i) > nb_pixels_2 and nb_pixels_98
            hist_cum = 0
            for i in range(len(histogram)):
                #print "i", i
                if hist_cum > nb_pixels_2 and self.x_min == 0 :
                    self.x_min = i + rasterMin
                if hist_cum > nb_pixels_98 :
                    self.x_max = i + rasterMin
                    break;
                hist_cum += histogram[i]
            print "self.x_min, self.x_max", self.x_min, self.x_max
            self.two_min = self.x_min
            self.ninety_eight_max = self.x_max
            print "self.two_min, self.ninety_eight_max", self.two_min, self.ninety_eight_max

            return histogram


    def display_histogram(self, filename, band, color, name):
        self.color = color
        self.name = name
        histogram = self.get_GDAL_histogram(filename, band)
        #print histogram
        self.t = range(0, len(histogram))
        self.s = histogram
        self.draw_histogram()
        self.axes.figure.canvas.mpl_connect('button_press_event', self.on_press)
        self.axes.figure.canvas.mpl_connect('button_release_event', self.on_release)
        self.draw_min_max_percent()
        
        
    def draw_histogram(self):
        if self.t and self.s and self.color and self.name:
            self.axes.plot(self.t, self.s, self.color)
            xtext = self.axes.set_xlabel('Valeur') # returns a Text instance
            ytext = self.axes.set_ylabel('Nombre')
            self.axes.set_title(self.name)
            
    def draw_reset_percent(self):
        print "draw reset"
        self.axes.clear()
        self.draw_histogram()
        if self.two_min and self.ninety_eight_max:
             print "self.two_min, self.ninety_eight_max", self.two_min, self.ninety_eight_max
             self.axes.axvline(x=self.two_min,c="red",linewidth=2,zorder=0, clip_on=False)
             self.axes.axvline(x=self.ninety_eight_max,c="red",linewidth=2,zorder=0, clip_on=False)
        self.axes.figure.canvas.draw()



    def draw_min_max_percent(self):
        if self.x_min and self.x_max:
            print "self.x_min, self.x_max", self.x_min, self.x_max
            self.axes.axvline(x=self.x_min,c="red",linewidth=2,zorder=0, clip_on=False)
            self.axes.axvline(x=self.x_max,c="red",linewidth=2,zorder=0, clip_on=False)
        
        
    def on_press(self, event):
        """
        Check if the clicked point is nearer from x_min than from x_max
        """
        print 'press'
        x = event.xdata
        # take x_min
        if abs(x-self.x_min) < abs(x-self.x_max):
            self.change_min = True
        else:
            self.change_min = False
        #self.valueChanged.emit()
        self.emit( QtCore.SIGNAL("valueChanged()") )
        
        #self.x_min = 

    def on_release(self, event):
        """
        Check
        """
        print 'release'
        if self.change_min :
            self.x_min = event.xdata
        else:
            self.x_max = event.xdata
        
        self.axes.clear()
        self.draw_histogram()
        #self.axes.plot(self.t, self.s, self.color)
        #self.axes.plot(self.x1, self.x1, 'g^')
        self.draw_min_max_percent()
        #self.axes.vlines(self.x1, [0], self.x1)
#         self.axes.set_xlabel('time (s)')
#         self.axes.set_title('Vertical lines demo')
        
        self.axes.figure.canvas.draw()
        
        print self.x_min, self.x_max


class ApplicationWindow(QtGui.QMainWindow):
    __pyqtSignals__ = ("valueChanged(PyQt_PyObject)", "threshold(PyQt_PyObject)")
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        self.setWindowTitle("application main window")

        self.file_menu = QtGui.QMenu('&File', self)
        self.file_menu.addAction('&Quit', self.fileQuit,
                                 QtCore.Qt.CTRL + QtCore.Qt.Key_Q)
        self.menuBar().addMenu(self.file_menu)

        self.help_menu = QtGui.QMenu('&Help', self)
        self.menuBar().addSeparator()
        self.menuBar().addMenu(self.help_menu)

        self.help_menu.addAction('&About', self.about)

        self.main_widget = QtGui.QWidget(self)

        l = QtGui.QVBoxLayout(self.main_widget)
        self.sc_1 = MyMplCanvas(self.main_widget, width=5, height=4, dpi=100)
        #self.sc_1.valueChanged.connect( self.valueChanged )
        QtCore.QObject.connect( self.sc_1, QtCore.SIGNAL( "valueChanged()" ), self.valueChanged )
        self.sc_2 = MyMplCanvas(self.main_widget, width=5, height=4, dpi=100)
        self.sc_3 = MyMplCanvas(self.main_widget, width=5, height=4, dpi=100)
        l.addWidget(self.sc_1)
        l.addWidget(self.sc_2)
        l.addWidget(self.sc_3)
        b = QtGui.QPushButton("Remise à zéro")
        b.clicked.connect(self.reset)
        l.addWidget(b)
        seuil = QtGui.QPushButton("Seuillage")
        seuil.clicked.connect(self.seuillage)
        l.addWidget(seuil)
        


        timeBegin = time.time()
        layer = '/home/amondot/TerreImage/Data/SPOT_2.5_L93/toulouse_2_5m_l93.tif' #'/home/amondot/otb-logo.png'
        #file_pointer = rasterIO.opengdalraster(layer.get_source())
        # pir, red, green
        #band = rasterIO.readrasterband(file_pointer, 1)
        #sc_1.compute_initial_figure(band, 'r')
        self.sc_1.display_histogram(layer, 1, 'r', "Plan R : BS PIR")
        #band = rasterIO.readrasterband(file_pointer, 2)
        #sc_2.compute_initial_figure(band, 'g')
        self.sc_2.display_histogram(layer, 2, 'g', "Plan V : BS R")
        #band = rasterIO.readrasterband(file_pointer, 3)
        #sc_3.compute_initial_figure(band, 'b')
        self.sc_3.display_histogram(layer, 3, 'b', "Plan B : BS V")
        #dc.compute_initial_figure()
        #dc.compute_initial_figure()
        timeEnd = time.time()
        timeExec = timeEnd - timeBegin


        self.main_widget.setFocus()
        self.setCentralWidget(self.main_widget)

        self.statusBar().showMessage("All hail matplotlib!", 2000)

    def fileQuit(self):
        self.close()

    def closeEvent(self, ce):
        self.fileQuit()
        
    def reset(self):
        print "resset"
        self.sc_1.draw_reset_percent()
        self.sc_2.draw_reset_percent()
        self.sc_3.draw_reset_percent()
        
    def seuillage(self):
        forms = []
        forms.append( "if((im1b1>" + str(self.sc_1.x_min) + ") and (im1b1<" + str(self.sc_1.x_max) + ")), im1b1, 0)" )
        forms.append( "if((im2b1>" + str(self.sc_2.x_min) + ") and (im2b1<" + str(self.sc_2.x_max) + ")), im2b1, 0)" )
        forms.append( "if((im3b1>" + str(self.sc_3.x_min) + ") and (im3b1<" + str(self.sc_3.x_max) + ")), im3b1, 0)" )
        print forms
        #emit signal
        
    def valueChanged(self):
        print "value changed"
        

    def about(self):
        QtGui.QMessageBox.about(self, "About",
"""embedding_in_qt4.py example
Copyright 2005 Florent Rougon, 2006 Darren Dale

This program is a simple example of a Qt4 application embedding matplotlib
canvases.

It may be used and modified with no restriction; raw copies as well as
modified versions may be distributed without limitation."""
)


qApp = QtGui.QApplication(sys.argv)

aw = ApplicationWindow()
aw.setWindowTitle("%s" % progname)
aw.show()


sys.exit(qApp.exec_())
#qApp.exec_()


